use std::path::{Path, PathBuf};


#[cfg(test)]
pub mod tests;
pub mod store;
pub mod song;
pub mod playlist;
pub mod query;

pub type Result<T> = anyhow::Result<T>;

#[derive(Debug, Clone)]
pub struct Manifest<ST: store::BaseStore> {
    store: Box<ST>,
}

impl<ST: store::BaseStore + Clone> Manifest<ST> {
    pub fn new(p: &Path) -> Result<Self>{
        let mut store = ST::empty();
        if p.exists() {
            store.load_from(p)?;
        } else {
            store.save_to(p)?; 
        }
        store.save_original_path(p);
        Ok(Self { 
            store: Box::new(store)
        })
    }
    pub fn store(&self) -> &ST {
        self.store.as_ref()
    }
    pub fn store_mut(&mut self) -> &mut ST {
        self.store.as_mut()
    }
    pub fn save(&self) -> Result<()> {
        self.store().save_to(self.store().get_original_path())?;
        Ok(())
    }
    pub fn load(&mut self) -> Result<()> {
        let p = self.store().get_original_path().to_path_buf();
        self.store_mut().load_from(&p)?;
        Ok(())
    }
    pub fn convert_to<ST2: store::BaseStore>(&self) -> ST2 {
        let songs = self.store().get_songs().clone();
        let playlists = self.store().get_playlists().clone();
        let mut st2 = ST2::empty();
        *st2.get_songs_mut() = songs;
        *st2.get_playlists_mut() = playlists;
        st2
    }

    pub fn convert_and_save_to<ST2: store::BaseStore>(&self, path: &Path) -> Result<()> {
        let st2 = self.convert_to::<ST2>();
        std::fs::write(path, st2.to_bytes()?)?;
        Ok(())
    }
    
    pub fn get_song_as_path(&self, sid: uuid::Uuid) -> Result<PathBuf> {
        let ext = &xmpd_settings::Settings::get()?.tooling.song_format;
        let mut p = xmpd_cliargs::CLIARGS.cache_path().into_std_path_buf();
        p.push("songs");
        p.push(sid.to_string());
        p.set_extension(ext);
        Ok(p)
    }
}









